module Memiescape.Loader where

import Data.Maybe (Maybe(..), maybe)
import Data.Tuple (fst, snd)
import Data.Tuple.Nested ((/\))
import DOM.Simple.Console (log2)
import FFI.Simple ((..))
import Prelude (($), (<$>), const, discard, pure, show, unit)
import Reactix as R
import Reactix.DOM.HTML as H
import Web.File.Blob (Blob)
import Web.File.File as WFF
import Web.File.FileList (FileList, item)

import Memiescape.Types

thisModule :: String
thisModule = "Memiescape.Loader"

type LoaderProps = (
  file :: R.State (Maybe (Record FileNameWithBlob))
  )

loader :: Record LoaderProps -> R.Element
loader props = R.createElement loaderCpt props []

loaderCpt :: R.Component LoaderProps
loaderCpt = R.hooksComponentWithModule thisModule "loader" cpt
  where
    cpt { file: (file /\ setFile) } _ = do
      pure $ H.div {} [
        H.div {} [
            H.label {} [ H.text "Load a philomemy" ]
          , H.input { on: { change: onChange }
                    , type: "file" }
          ]
          , H.div {} [ H.text $ maybe "" (\f -> f.name) file ]
        ]
      where
        onChange e = do
          let mF = inputFileNameWithBlob 0 e
          log2 "[loader] mF" mF
          setFile $ const mF

inputFileNameWithBlob :: forall e. Int -> e -> Maybe (Record FileNameWithBlob)
inputFileNameWithBlob n e = case ff of
    Nothing -> Nothing
    Just f  -> Just { blob: WFF.toBlob f, name: WFF.name f }
  where
    ff = inputFile n e

inputFile :: forall e. Int -> e -> Maybe WFF.File
inputFile n e = item n $ ((el .. "files") :: FileList)
  where
    el = e .. "target"
