module Memiescape.Types where

import Prelude

import Data.Argonaut (class DecodeJson, (.:), decodeJson)
import Data.Argonaut.Decode.Error (JsonDecodeError(..))
import Data.Either (Either(..))
import Data.Tuple (Tuple(..))
import Web.File.Blob (Blob)

type FileNameWithBlob = (
    blob :: Blob
  , name :: String
  )

newtype Phylomeme = Phylomeme {
    _subraph_cnt     :: Int
  , bb               :: String  -- TODO "0,0,8957.4,12641"
  , color            :: String
  , directed         :: Boolean
  , edges            :: Array PhyloEdge
  , fontsize         :: Int  -- TODO String in the sample JSON file
  , label            :: String
  , labelloc         :: String  -- TODO
  , lheight          :: Number  -- TODO String in the sample JSON file
  , lp               :: String  -- TODO "4478.7,12620"
  , lwidth           :: Number  -- TODO String in the sample JSON file
  , name             :: String
  , nodesep          :: String  -- TODO
  , objects          :: Array PhyloObject
  , overlap          :: String
  , phyloBranches    :: Int  -- TODO String in the sample JSON file
  , phyloDocs        :: Number  -- TODO String in the sample JSON file
  , phyloFoundations :: Int  -- TODO String in the sample JSON file
  , phyloGroups      :: Int  -- TODO String in the sample JSON file
  , phyloPeriods     :: Int  -- TODO String in the sample JSON file
  , phyloTerms       :: Int  -- TODO String in the sample JSON file
  , phyloTermsFreq   :: Array (Tuple Int Number)  -- TODO String in the sample JSON file
  , rank             :: String
  , ranksep          :: Int  -- TODO String in the sample JSON file
  , ratio            :: String
  , splines          :: String
  , strict           :: Boolean
  , style            :: String
  }

data PhyloEdge = PhyloBranchLink {
    _gvid      :: Int
  , arrowhead  :: String
  , color      :: String
  , edgeType   :: String  -- NOTE "branchLink"
  , head       :: Int
  , pos        :: String  -- TODO "e,6347.4,1337.2 6347.4,1408.8 6347.4,1388.4 6347.4,1367.7 6347.4,1347.3"
  , tail       :: Int
  , width      :: Int  -- TODO String in the sample JSON file
  } | PhyloLink {
    _gvid      :: Int
  , color      :: String
  , constraint :: Boolean  -- TODO String in the sample JSON file
  , edgeType   :: String  -- NOTE "link"
  , head       :: Int
  , lbl        :: Number  -- TODO String in the sample JSON file
  , penwidth   :: Int  -- TODO String in the sample JSON file
  , pos        :: String  -- TODO "e,6347.4,1337.2 6347.4,1408.8 6347.4,1388.4 6347.4,1367.7 6347.4,1347.3"
  , tail       :: Int
  , width      :: Int  -- TODO String in the sample JSON file
  }

data PhyloObject = PhyloBranch {
    _gvid      :: Int
  , age        :: Number  -- TODO String in the sample JSON file
  , bId        :: Int  -- TODO String in the sample JSON file
  , birth      :: Number  -- TODO String in the sample JSON file
  , branchId   :: String  -- TODO "0 3 1 1 1 3 2 1 1 1 1"
  , branch_x   :: Number  -- TODO String in the sample JSON file
  , branch_y   :: Number  -- TODO String in the sample JSON file
  , fillcolor  :: String
  , fontname   :: String
  , fontsize   :: Int  -- TODO String in the sample JSON file
  , height     :: Number  -- TODO String in the sample JSON file
  , label      :: String
  , nodeType   :: String  -- NOTE "branch"
  , pos        :: String  -- TODO "8491.4,220"
  , shape      :: String
  , size       :: Number  -- TODO String in the sample JSON file
  , style      :: String
  , width      :: Number  -- TODO String in the sample JSON file
  } | PhyloGroup {
    _gvid      :: Int
  , bId        :: Int  -- TODO String in the sample JSON file
  , branchId   :: String  -- TODO "0 3 1 1 1 3 2 1 1 1 1"
  , fontname   :: String
  , foundation :: String -- TODO "\"8 | 12\""
  , from       :: Int  -- TODO String in the sample JSON file
  , height     :: Number  -- TODO String in the sample JSON file
  , label      :: String
  , lbl        :: String
  , nodeType   :: String  -- NOTE "group"
  , penwidth   :: Int  -- TODO String in the sample JSON file
  , pos        :: String  -- TODO "8491.4,220"
  , role       :: String  -- TODO "\"1.0 | 0.0\""
  , shape      :: String
  , support    :: Int  -- TODO String in the sample JSON file
  , to         :: Int  -- TODO String in the sample JSON file
  , width      :: Number  -- TODO String in the sample JSON file
  } | PhyloPeriod {
    _gvid      :: Int
  , fontsize   :: Int  -- TODO String in the sample JSON file
  , from       :: Int  -- TODO String in the sample JSON file
  , height     :: Number  -- TODO String in the sample JSON file
  , label      :: String
  , nodeType   :: String  -- NOTE "period"
  , pos        :: String  -- TODO "8491.4,220"
  , shape      :: String
  , to         :: Int  -- TODO String in the sample JSON file
  , width      :: Number  -- TODO String in the sample JSON file
  }

instance decodeJsonPhyloObject :: DecodeJson PhyloObject where
  decodeJson json = do
    obj <- decodeJson json
    gvId <- obj     .: "_gvid"
    nodeType <- obj .: "nodeType"

    case nodeType of
      "branch" -> do
        age <- obj       .: "age"
        bId <- obj       .: "bId"
        birth <- obj     .: "birth"
        branchId <- obj  .: "branchId"
        branch_x <- obj  .: "branch_x"
        branch_y <- obj  .: "branch_y"
        fillcolor <- obj .: "fillcolor"
        fontname <- obj  .: "fontname"
        fontsize <- obj  .: "fontsize"
        height <- obj    .: "height"
        label <- obj     .: "label"
        pos <- obj       .: "pos"
        shape <- obj     .: "shape"
        size <- obj      .: "size"
        style <- obj     .: "style"
        width <- obj     .: "width"
        pure $ PhyloBranch {
            _gvid: gvId
          , age
          , bId
          , birth
          , branchId
          , branch_x
          , branch_y
          , fillcolor
          , fontname
          , fontsize
          , height
          , label
          , nodeType
          , pos
          , shape
          , size
          , style
          , width
          }
      "group" -> do
        bId <- obj        .: "bId"
        branchId <- obj   .: "branchId"
        fontname <- obj   .: "fontname"
        foundation <- obj .: "foundation"
        from <- obj       .: "from"
        height <- obj     .: "height"
        label <- obj      .: "label"
        lbl <- obj        .: "lbl"
        penwidth <- obj   .: "penwidth"
        pos <- obj        .: "pos"
        role <- obj       .: "role"
        shape <- obj      .: "shape"
        support <- obj    .: "support"
        to <- obj         .: "to"
        width <- obj      .: "width"
        pure $ PhyloGroup {
            _gvid: gvId
          , bId
          , branchId
          , fontname
          , foundation
          , from
          , height
          , label
          , lbl
          , nodeType
          , penwidth
          , pos
          , role
          , shape
          , support
          , to
          , width
          }
      "period" -> do
        fontsize <- obj .: "fontsize"
        from <- obj     .: "from"
        height <- obj   .: "height"
        label <- obj    .: "label"
        pos <- obj      .: "pos"
        shape <- obj    .: "shape"
        to <- obj       .: "to"
        width <- obj    .: "width"
        pure $ PhyloPeriod {
            _gvid: gvId
          , fontsize
          , from
          , height
          , label
          , nodeType
          , pos
          , shape
          , to
          , width
          }
      _ -> Left $ TypeMismatch $ "Unknown nodeType: " <> nodeType
